@@ -41,7 +41,7 @@ import rasterio.enums
 import rasterio.path
 
 __all__ = ['band', 'open', 'pad', 'Env']
-__version__ = "1.1.5"
+__version__ = "1.1.7"
 __gdal_version__ = gdal_version()
 
 # Rasterio attaches NullHandler to the 'rasterio' logger and its
@@ -85,9 +85,12 @@ def open(fp, mode='r', driver=None, width=None, height=None, count=None,
         sequentially until a match is found. When multiple drivers are
         available for a format such as JPEG2000, one of them can be
         selected by using this keyword argument.
-    width, height : int, optional
-        The numbers of rows and columns of the raster dataset. Required
-        in 'w' or 'w+' modes, they are ignored in 'r' or 'r+' modes.
+    width : int, optional
+        The number of columns of the raster dataset. Required in 'w' or
+        'w+' modes, it is ignored in 'r' or 'r+' modes.
+    height : int, optional
+        The number of rows of the raster dataset. Required in 'w' or
+        'w+' modes, it is ignored in 'r' or 'r+' modes.
     count : int, optional
         The count of dataset bands. Required in 'w' or 'w+' modes, it is
         ignored in 'r' or 'r+' modes.
