@@ -1,15 +1,13 @@
-import sys
 import os
-import logging
-import numpy as np
+
 from click.testing import CliRunner
+import numpy as np
+import pytest
 
 import rasterio
 from rasterio.rio.main import main_group
 
 
-logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
-
 TEST_BBOX = [-11850000, 4804000, -11840000, 4808000]
 
 
@@ -17,11 +15,12 @@ def bbox(*args):
     return ' '.join([str(x) for x in args])
 
 
-def test_clip_bounds(runner, tmpdir):
+@pytest.mark.parametrize("bounds", [bbox(*TEST_BBOX)])
+def test_clip_bounds(runner, tmpdir, bounds):
     output = str(tmpdir.join('test.tif'))
     result = runner.invoke(
-        main_group,
-        ['clip', 'tests/data/shade.tif', output, '--bounds', bbox(*TEST_BBOX)])
+        main_group, ["clip", "tests/data/shade.tif", output, "--bounds", bounds]
+    )
     assert result.exit_code == 0
     assert os.path.exists(output)
 
@@ -29,6 +28,29 @@ def test_clip_bounds(runner, tmpdir):
         assert out.shape == (419, 173)
 
 
+@pytest.mark.parametrize("bounds", [bbox(*TEST_BBOX)])
+def test_clip_bounds_with_complement(runner, tmpdir, bounds):
+    output = str(tmpdir.join("test.tif"))
+    result = runner.invoke(
+        main_group,
+        [
+            "clip",
+            "tests/data/shade.tif",
+            output,
+            "--bounds",
+            bounds,
+            "--with-complement",
+        ],
+    )
+    assert result.exit_code == 0
+    assert os.path.exists(output)
+
+    with rasterio.open(output) as out:
+        assert out.shape == (419, 1047)
+        data = out.read()
+        assert (data[420:, :] == 255).all()
+
+
 def test_clip_bounds_geographic(runner, tmpdir):
     output = str(tmpdir.join('test.tif'))
     result = runner.invoke(
@@ -106,9 +128,16 @@ def test_clip_overwrite_with_option(runner, tmpdir):
     assert result.exit_code == 0
 
     result = runner.invoke(
-        main_group, [
-        'clip', 'tests/data/shade.tif', output, '--bounds', bbox(*TEST_BBOX),
-        '--overwrite'])
+        main_group,
+        [
+            "clip",
+            "tests/data/shade.tif",
+            output,
+            "--bounds",
+            bbox(*TEST_BBOX),
+            "--overwrite",
+        ],
+    )
     assert result.exit_code == 0
 
 
@@ -258,3 +287,20 @@ def test_convert_overwrite_with_option(runner, tmpdir):
         'convert', 'tests/data/RGB.byte.tif', '-o', outputname, '-f', 'JPEG',
         '--overwrite'])
     assert result.exit_code == 0
+
+
+def test_convert_no_input(runner, tmpdir):
+    """Test fix of issue1985"""
+    outputname = str(tmpdir.join("test.tif"))
+    result = runner.invoke(main_group, ["convert", "-o", outputname, "-f", "JPEG"])
+    assert result.exit_code == 2
+
+
+def test_convert_no_input_overwrite(runner, tmpdir):
+    """Test fix of issue1985"""
+    outputname = str(tmpdir.join("test.tif"))
+    result = runner.invoke(
+        main_group, ["convert", "--overwrite", outputname, "-f", "JPEG"]
+    )
+    assert result.exit_code == 2
+    assert "Insufficient inputs" in result.output
