@@ -13,7 +13,12 @@ from rasterio.control import GroundControlPoint
 from rasterio.crs import CRS
 from rasterio.enums import Resampling
 from rasterio.env import GDALVersion
-from rasterio.errors import (GDALBehaviorChangeException, CRSError, GDALVersionError)
+from rasterio.errors import (
+    GDALBehaviorChangeException,
+    CRSError,
+    GDALVersionError,
+    TransformError,
+)
 from rasterio.warp import (
     reproject,
     transform_geom,
@@ -101,12 +106,12 @@ WGS84_crs = CRS.from_epsg(4326)
 
 def test_transform_src_crs_none():
     with pytest.raises(CRSError):
-        transform(None, WGS84_crs, [], [])
+        transform(None, WGS84_crs, [1], [1])
 
 
 def test_transform_dst_crs_none():
     with pytest.raises(CRSError):
-        transform(WGS84_crs, None, [], [])
+        transform(WGS84_crs, None, [1], [1])
 
 
 def test_transform_bounds_src_crs_none():
@@ -217,19 +222,23 @@ def test_transform_bounds__esri_wkt():
     )
 
 
-def test_transform_bounds_densify():
+@pytest.mark.parametrize(
+    "density,expected",
+    [
+        (0, (-1684649.41338, -350356.81377, 1684649.41338, 2234551.18559)),
+        (100, (-1684649.41338, -555777.79210, 1684649.41338, 2234551.18559)),
+    ],
+)
+def test_transform_bounds_densify(density, expected):
     # This transform is non-linear along the edges, so densification produces
     # a different result than otherwise
     src_crs = CRS.from_epsg(4326)
-    dst_crs = CRS.from_epsg(2163)
-    assert np.allclose(
-        transform_bounds(src_crs, dst_crs, -120, 40, -80, 64, densify_pts=0),
-        (-1684649.41338, -350356.81377, 1684649.41338, 2234551.18559),
+    dst_crs = CRS.from_proj4(
+        "+proj=laea +lat_0=45 +lon_0=-100 +x_0=0 +y_0=0 +a=6370997 +b=6370997 +units=m +no_defs"
     )
-
     assert np.allclose(
-        transform_bounds(src_crs, dst_crs, -120, 40, -80, 64, densify_pts=100),
-        (-1684649.41338, -555777.79210, 1684649.41338, 2234551.18559),
+        expected,
+        transform_bounds(src_crs, dst_crs, -120, 40, -80, 64, densify_pts=density),
     )
 
 
@@ -1092,6 +1101,59 @@ def test_reproject_resampling(path_rgb_byte_tif, method):
 
     assert np.count_nonzero(out) in expected[method]
 
+@pytest.mark.parametrize("test3d,count_nonzero", [(True, 1309625), (False, 437686)])
+def test_reproject_array_interface(test3d, count_nonzero, path_rgb_byte_tif):
+    class DataArray:
+        def __init__(self, data):
+            self.data = data
+
+        def __array__(self, dtype=None):
+            return self.data
+
+        @property
+        def dtype(self):
+            return self.data.dtype
+
+    with rasterio.open(path_rgb_byte_tif) as src:
+        if test3d:
+            source = DataArray(src.read())
+        else:
+            source = DataArray(src.read(1))
+    out = DataArray(np.empty(source.data.shape, dtype=np.uint8))
+    reproject(
+        source,
+        out,
+        src_transform=src.transform,
+        src_crs=src.crs,
+        src_nodata=src.nodata,
+        dst_transform=DST_TRANSFORM,
+        dst_crs="EPSG:3857",
+        dst_nodata=99,
+    )
+    assert isinstance(out, DataArray)
+    assert np.count_nonzero(out.data[out.data != 99]) == count_nonzero
+
+
+@pytest.mark.parametrize("test3d,count_nonzero", [(True, 1309625), (False, 437686)])
+def test_reproject_masked(test3d, count_nonzero, path_rgb_byte_tif):
+    with rasterio.open(path_rgb_byte_tif) as src:
+        if test3d:
+            source = src.read(masked=True)
+        else:
+            source = src.read(1, masked=True)
+    out = np.empty(source.shape, dtype=np.uint8)
+    reproject(
+        source,
+        out,
+        src_transform=src.transform,
+        src_crs=src.crs,
+        dst_transform=DST_TRANSFORM,
+        dst_crs="EPSG:3857",
+        dst_nodata=99,
+    )
+    assert np.ma.is_masked(source)
+    assert np.count_nonzero(out[out != 99]) == count_nonzero
+
 
 @pytest.mark.parametrize("method", SUPPORTED_RESAMPLING)
 def test_reproject_resampling_alpha(method):
@@ -1628,3 +1690,29 @@ def test_reproject_init_dest_nodata():
         src_nodata=0, init_dest_nodata=False
     )
     assert destination.all()
+
+
+def test_empty_transform_inputs():
+    """Check for fix of #1952"""
+    assert ([], []) == rasterio.warp.transform(
+        "EPSG:3857", "EPSG:4326", [], [], zs=None
+    )
+
+
+def test_empty_transform_inputs_z():
+    """Check for fix of #1952"""
+    assert ([], [], []) == rasterio.warp.transform(
+        "EPSG:3857", "EPSG:4326", [], [], zs=[]
+    )
+
+
+def test_empty_transform_inputs_length():
+    """Get an exception of inputs have different lengths"""
+    with pytest.raises(TransformError):
+        rasterio.warp.transform("EPSG:3857", "EPSG:4326", [1], [1, 2])
+
+
+def test_empty_transform_inputs_length_z():
+    """Get an exception of inputs have different lengths"""
+    with pytest.raises(TransformError):
+        rasterio.warp.transform("EPSG:3857", "EPSG:4326", [1, 2], [1, 2], zs=[0])
